@@ -16,6 +16,7 @@ class Generator:
         TokenType.HAVING,
         TokenType.ORDER,
         TokenType.UNION,
+        TokenType.WITH,
     }
 
     TRANSFORMS = {
@@ -39,6 +40,7 @@ class Generator:
         exp.DateDiff: lambda self, e: f"DATE_DIFF({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
         exp.DateStrToDate: lambda self, e: f"DATE_STR_TO_DATE({self.sql(e, 'this')})",
         exp.Initcap: lambda self, e: f"INITCAP({self.sql(e, 'this')})",
+        exp.JSONPath: lambda self, e: f"JSON_PATH({self.sql(e, 'this')}, {self.sql(e, 'path')})",
         exp.StrPosition: lambda self, e: f"STR_POSITION({csv(self.sql(e, 'this'), self.sql(e, 'substr'), self.sql(e, 'position'))})",
         exp.StrToTime: lambda self, e: f"STR_TO_TIME({self.sql(e, 'this')}, {self.sql(e, 'format')})",
         exp.StrToUnix: lambda self, e: f"STR_TO_UNIX({self.sql(e, 'this')}, {self.sql(e, 'format')})",
@@ -66,10 +68,11 @@ class Generator:
         self.unsupported_messages = []
         self._indent = opts.get('indent', 4)
         self._level = 0
+        self.opts = opts
 
     def generate(self, expression):
         self.unsupported_messages = []
-        sql = self.sql(expression)
+        sql = self.sql(expression).strip()
 
         if self.unsupported_level == ErrorLevel.IGNORE:
             return sql
@@ -104,10 +107,9 @@ class Generator:
         return f"({self.sep('')}{this_sql}{self.seg(')', sep='')}"
 
     def no_format(self, func):
-        pretty = self.pretty
-        self.pretty = None
+        self.pretty = False
         result = func()
-        self.pretty = pretty
+        self.pretty = self.opts.get('pretty')
         return result
 
     def indent_newlines(self, sql, skip_first=False):
@@ -167,7 +169,7 @@ class Generator:
             for e in expression.args['expressions']
         )
 
-        return f"WITH {sql}{self.sep()}{self.sql(expression, 'this')}"
+        return f"WITH {sql}{self.sep()}{self.indent(self.sql(expression, 'this'))}"
 
     def drop_sql(self, expression):
         this = self.sql(expression, 'this')
@@ -297,15 +299,22 @@ class Generator:
     def case_sql(self, expression):
         pad = self.pad + 2
 
+        this = self.sql(expression, 'this')
+        this = f" {this}" if this else ''
+
         ifs = [
-            self.seg(f"WHEN {self.sql(e, 'this')} THEN {self.sql(e, 'true')}", pad=pad)
+            f"WHEN {self.sql(e, 'this')} THEN {self.sql(e, 'true')}"
             for e in expression.args['ifs']
         ]
 
         if expression.args.get('default') is not None:
-            ifs.append(self.seg(f"ELSE {self.sql(expression, 'default')}", pad=pad))
+            ifs.append(f"ELSE {self.sql(expression, 'default')}")
 
-        return f"CASE{''.join(ifs)}{self.seg('END', pad=self.pad)}"
+        self.pretty = self.opts.get('pretty')
+        ifs = ''.join(self.seg(e, pad=pad) for e in ifs)
+        case = f"CASE{this}{ifs}{self.seg('END', pad=self.pad)}"
+        self.pretty = False
+        return case
 
     def decimal_sql(self, expression):
         if isinstance(expression, Token):
@@ -339,15 +348,14 @@ class Generator:
         return f"NOT {self.sql(expression, 'this')}"
 
     def alias_sql(self, expression):
-        this_sql = self.sql(expression, 'this')
         to_sql = self.sql(expression, 'alias')
         to_sql = f" AS {to_sql}" if to_sql else ''
 
         if expression.args['this'].token_type in self.BODY_TOKENS:
             if self.pretty:
                 return f"{self.wrap(expression)}{to_sql}"
-            return f"({this_sql}){to_sql}"
-        return f"{this_sql}{to_sql}"
+            return f"({self.sql(expression, 'this')}){to_sql}"
+        return f"{self.sql(expression, 'this')}{to_sql}"
 
     def and_sql(self, expression):
         return self.binary(expression, 'AND', newline=self.pretty)
