@@ -29,6 +29,7 @@ class Parser:
         'DATE_STR_TO_DATE': lambda args: exp.DateStrToDate(this=args[0]),
         'IF': lambda args: exp.If(this=args[0], true=args[1], false=list_get(args, 2)),
         'INITCAP': lambda args: exp.Initcap(this=args[0]),
+        'JSON_PATH': lambda args: exp.JSONPath(this=args[0], path=args[1]),
         'STR_POSITION': lambda args: exp.StrPosition(this=args[0], substr=args[1], position=list_get(args, 2)),
         'STR_TO_TIME': lambda args: exp.StrToTime(this=args[0], format=args[1]),
         'STR_TO_UNIX': lambda args: exp.StrToUnix(this=args[0], format=args[1]),
@@ -99,8 +100,8 @@ class Parser:
 
     def __init__(self, **opts):
         self.functions = {**self.FUNCTIONS, **(opts.get('functions') or {})}
-        self.error_level = opts.get('error_level', ErrorLevel.RAISE)
-        self.error_message_context = opts.get('error_message_context', 50)
+        self.error_level = opts.get('error_level') or ErrorLevel.RAISE
+        self.error_message_context = opts.get('error_message_context') or 50
         self.reset()
 
     def reset(self):
@@ -125,16 +126,10 @@ class Parser:
             self._index = -1
             self._advance()
             self._tokens = tokens
+            expressions.append(self._parse_statement())
 
-            try:
-                expressions.append(self._parse_statement())
-                if self._index < len(self._tokens):
-                    self.raise_error('Invalid expression / Unexpected token')
-            except ParseError as e:
-                if self.error_level == ErrorLevel.WARN:
-                    logging.error(e)
-                elif self.error_level == ErrorLevel.RAISE:
-                    raise e
+            if self._index < len(self._tokens):
+                self.raise_error('Invalid expression / Unexpected token')
 
         for expression in expressions:
             if not isinstance(expression, exp.Expression):
@@ -154,7 +149,10 @@ class Parser:
         highlight = self.code[start:end]
         end_context = self.code[end:end + self.error_message_context]
         self.error = ParseError(f"{message}\n  {start_context}\033[4m{highlight}\033[0m{end_context}")
-        raise self.error
+        if self.error_level == ErrorLevel.RAISE:
+            raise self.error
+        if self.error_level == ErrorLevel.WARN:
+            logging.error(self.error)
 
     def _find_token(self, token, code):
         line = 0
@@ -193,20 +191,19 @@ class Parser:
             return None
 
     def _parse_statement(self):
+        if self._curr is None:
+            return None
+
         if self._match(TokenType.CREATE):
             return self._parse_create()
 
         if self._match(TokenType.DROP):
             return self._parse_drop()
 
-        if self._match(TokenType.WITH):
-            expressions = self._parse_csv(self._parse_cte)
-            return exp.CTE(this=self._parse_select(), expressions=expressions)
+        cte = self._parse_cte()
 
-        select = self._parse_select()
-
-        if select:
-            return select
+        if cte:
+            return cte
 
         return self._parse_expression()
 
@@ -279,15 +276,26 @@ class Parser:
         return None
 
     def _parse_cte(self):
-        if not self._match(TokenType.IDENTIFIER, TokenType.VAR):
-            self.raise_error('Expected alias after WITH')
+        if not self._match(TokenType.WITH):
+            return self._parse_select()
 
-        alias = self._prev
+        expressions = []
 
-        if not self._match(TokenType.ALIAS):
-            self.raise_error('Expected AS after WITH')
+        while True:
+            if not self._match(TokenType.IDENTIFIER, TokenType.VAR):
+                self.raise_error('Expected alias after WITH')
 
-        return self._parse_table(alias=alias)
+            alias = self._prev
+
+            if not self._match(TokenType.ALIAS):
+                self.raise_error('Expected AS after WITH')
+
+            expressions.append(self._parse_table(alias=alias))
+
+            if not self._match(TokenType.COMMA):
+                break
+
+        return exp.CTE(this=self._parse_select(), expressions=expressions)
 
     def _parse_select(self):
         if not self._match(TokenType.SELECT):
@@ -373,11 +381,10 @@ class Parser:
             return unnest
 
         if self._match(TokenType.L_PAREN):
-            nested = self._parse_select()
+            expression = self._parse_cte()
 
             if not self._match(TokenType.R_PAREN):
                 self.raise_error('Expecting )')
-            expression = nested
         else:
             db = None
             table = self._match(TokenType.VAR, TokenType.IDENTIFIER)
@@ -541,6 +548,8 @@ class Parser:
         ifs = []
         default = None
 
+        expression = self._parse_expression()
+
         while self._match(TokenType.WHEN):
             this = self._parse_expression()
             self._match(TokenType.THEN)
@@ -553,7 +562,7 @@ class Parser:
         if not self._match(TokenType.END):
             self.raise_error('Expected END after CASE', self._prev)
 
-        return exp.Case(ifs=ifs, default=default)
+        return exp.Case(this=expression, ifs=ifs, default=default)
 
     def _parse_count(self):
         if not self._match(TokenType.L_PAREN):
@@ -618,13 +627,16 @@ class Parser:
                 self.raise_error('Expecting )', paren)
             return exp.Paren(this=this)
 
+        if self._curr is None:
+            return self.raise_error('Expecting expression')
+
         return self._parse_column()
 
     def _parse_column(self):
         db = None
         table = None
 
-        if self._curr.token_type in (TokenType.OVER, TokenType.R_PAREN, TokenType.R_BRACKET):
+        if self._curr.token_type in (TokenType.OVER, TokenType.R_PAREN, TokenType.R_BRACKET, TokenType.WHEN):
             return None
 
         self._advance()
